package com.jiuqi.db.impl;

import com.jiuqi.db.DBFunction;
import com.jiuqi.db.DBMetaData;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;
import java.util.List;

public class OracleDBMetaData implements DBMetaData {
   private static final String LeftParenthesis = "(";
   private static final String RightParenthesis = ")";
   private static final String comma = ",";
   private static final String BIT = "number(1)";
   private static final String NotNull = "not null";
   private static final String space = " ";
   private static final String DEF = " default ";
   private static final String TINYINT = "number(3)";
   private static final String SMALLINT = "number(5)";
   private static final String INTEGER = "number(10)";
   private static final String BIGINT = "number";
   private static final String FLOAT = "float";
   private static final String REAL = "real";
   private static final String DOUBLE = "float";
   private static final String NUMERIC = "number";
   private static final String DECIMAL = "number";
   private static final String CHAR = "char";
   private static final String VARCHAR = "varchar2";
   private static final String LONGVARCHAR = "long";
   private static final String TIMESTAMP = "date";
   private static final String VARBINARY = "raw";
   private static final String LONGVARBINARY = "long raw";
   private static final String BLOB = "blob";
   private static final String CLOB = "clob";
   private int maxColumnCount;
   private int maxRecordSize;
   private String defaultScheme;

   private void raise(String msg) throws SQLException {
      throw new SQLException(msg);
   }

   public OracleDBMetaData(DatabaseMetaData metaData) throws SQLException {
      this.maxColumnCount = metaData.getMaxColumnsInTable();
      this.maxRecordSize = 8000;
      this.defaultScheme = metaData.getUserName();
   }

   public int databaseType() {
      return 2;
   }

   public int maxColumnCount() {
      return this.maxColumnCount;
   }

   public int maxRecordSize() {
      return this.maxRecordSize;
   }

   public int maxIndexNameLength() {
      return 64;
   }

   public int maxFieldNameLength() {
      return 64;
   }

   public int maxTableNameLength() {
      return 64;
   }

   public int maxColumnByteSize(int sqlType) throws SQLException {
      int result = 0;
      switch (sqlType) {
         case -7:
            result = 1;
            break;
         case -6:
            result = 1;
            break;
         case -5:
            result = 8;
            break;
         case -4:
            result = 16;
            break;
         case -3:
            result = -1;
            break;
         case -2:
            result = -1;
            break;
         case -1:
            result = 16;
            break;
         case 1:
            result = -1;
            break;
         case 2:
            result = 17;
            break;
         case 3:
            result = 17;
            break;
         case 4:
            result = 4;
            break;
         case 5:
            result = 2;
            break;
         case 6:
            result = 8;
            break;
         case 7:
            result = 4;
            break;
         case 12:
            result = -1;
            break;
         case 93:
            result = 8;
            break;
         case 2004:
            result = 16;
            break;
         case 2005:
            result = 16;
      }

      if (result == 0) {
         this.raise("不支持的数据类型 " + Integer.toString(sqlType));
      }

      return result;
   }

   public String getFieldDDL(int sqlType, int size, int scale, boolean isNull, String defValue) throws SQLException {
      StringBuffer result = new StringBuffer();
      switch (sqlType) {
         case -7:
            result.append("number(1)");
            break;
         case -6:
            result.append("number(3)");
            break;
         case -5:
            result.append("number");
            break;
         case -4:
            result.append("long raw");
            break;
         case -3:
            if (size > 4000 || size <= 0) {
               this.raise("长度定义错误应在1~4000之间");
            }

            result.append("raw");
            result.append("(");
            result.append(size);
            result.append(")");
            break;
         case -1:
            result.append("long");
            break;
         case 1:
            if (size > 2000 || size <= 0) {
               this.raise("长度定义错误应在1~8000之间");
            }

            result.append("char");
            result.append("(");
            result.append(size);
            result.append(")");
            break;
         case 2:
            if (size < 0 || size > 38 || scale < 0 || scale > 38) {
               this.raise("精度或小数位定义错误");
            }

            result.append("number");
            result.append("(");
            result.append(size);
            result.append(",");
            result.append(scale);
            result.append(")");
            break;
         case 3:
            if (size < 0 || size > 38 || scale < 0 || scale > 38) {
               this.raise("精度或小数位定义错误");
            }

            result.append("number");
            result.append("(");
            result.append(size);
            result.append(",");
            result.append(scale);
            result.append(")");
            break;
         case 4:
            result.append("number(10)");
            break;
         case 5:
            result.append("number(5)");
            break;
         case 6:
            result.append("float");
            break;
         case 7:
            result.append("real");
            break;
         case 8:
            result.append("float");
            break;
         case 12:
            if (size > 4000 || size <= 0) {
               this.raise("长度定义错误应在1~8000之间");
            }

            result.append("varchar2");
            result.append("(");
            result.append(size);
            result.append(")");
            break;
         case 91:
         case 92:
         case 93:
            result.append("date");
            break;
         case 2004:
            result.append("blob");
            break;
         case 2005:
            result.append("clob");
      }

      if (result.length() == 0) {
         this.raise("不支持的数据类型" + Integer.toString(sqlType));
      }

      result.append(" ");
      if (defValue != null && !defValue.equals("")) {
         result.append(" default ");
         result.append(defValue);
      }

      result.append(" ");
      result.append(isNull ? " " : "not null");
      return result.toString();
   }

   public String getDropFieldPreSql(String tableName, String fieldName) {
      return null;
   }

   public String getDropFieldDDL(String tableName, String fieldName) {
      return "alter table " + tableName + " drop column " + fieldName;
   }

   public boolean getModifyFieldSqls(String tableName, String sFieldName, int sSqlType, int sSize, int sScale, boolean sIsNull, String sDefValue, String dFieldName, int dSqlType, int dSize, int dScale, boolean dIsNull, String dDefValue, List sqls) throws SQLException {
      StringBuffer sql = new StringBuffer();
      if (sSqlType == 2 && sSqlType == dSqlType && sFieldName.equals(dFieldName)) {
         if (dSize - dScale >= sSize - sScale && dScale >= sScale && dIsNull) {
            System.out.println("sSize:" + sSize + ", sScale:" + sScale + ", dSize:" + dSize + ", dScale:" + dScale);
            sql.append("alter table ");
            sql.append(tableName);
            sql.append(" modify(");
            sql.append(sFieldName);
            sql.append(" ");
            sql.append(this.getFieldDDL(dSqlType, dSize, dScale, dIsNull, dDefValue));
            sql.append(")");
            sqls.add(sql.toString());
            return true;
         }
      } else if (sSqlType == 4 && (dSqlType == 3 || sSqlType == 2) && sFieldName.equals(dFieldName)) {
         if (dSize > 10 && dIsNull) {
            sql.append("alter table ");
            sql.append(tableName);
            sql.append(" modify(");
            sql.append(sFieldName);
            sql.append(" ");
            sql.append(this.getFieldDDL(dSqlType, dSize, dScale, dIsNull, dDefValue));
            sql.append(")");
            sqls.add(sql.toString());
            return true;
         }
      } else if (sSqlType == 12 && dSqlType == sSqlType && sFieldName.equals(dFieldName) && dSize > sSize && dIsNull) {
         sql.append("alter table ");
         sql.append(tableName);
         sql.append(" modify(");
         sql.append(sFieldName);
         sql.append(" ");
         sql.append(this.getFieldDDL(dSqlType, dSize, dScale, dIsNull, dDefValue));
         sql.append(")");
         sqls.add(sql.toString());
         return true;
      }

      return false;
   }

   public boolean getDDLCausesTransactionCommit() {
      return true;
   }

   public boolean getDDLIgnoredInTransactions() {
      return false;
   }

   public String getDefaultScheme() {
      return this.defaultScheme;
   }

   public boolean getSupportDropColumn() {
      return true;
   }

   public DBFunction createDBFunction() {
      return new OracleDBFunction();
   }

   public boolean getTruncateNumber() {
      return false;
   }
}
